#! /usr/bin/perl

## $Id: configure.pl,v 1.10 2007/10/01 13:07:34 ianbezanson Exp $
##
## php-dropbox Configuration Script
##
## Purpose: Allows the user to setup and/or make changes to dropbox 
## configuration
##
## Methodology: - Check to see if `config.php` exists in current dir
##              - If exists, grab current config values
##              - If not exists, populate with DUMMY values
##              - User can then save new/updated config file
##

use POSIX;
use English;
use CGI qw(:standard);
use File::Path;   # for mkpath()
use FileHandle;

sub setValue ($$)
{
  $hashRef = shift;
  $thisVar = shift;
  $status = "active";
  system("clear");
  foreach my $items (sort { $hashRef->{$a}->{menuOrder} <=>
                          $hashRef->{$b}->{menuOrder} } (keys %{$hashRef}) )
  {
    my $menuNumber      = $hashRef->{$items}->{menuOrder};
    my $menuItem        = $hashRef->{$items}->{menuListing};
    my $defaultValue    = $hashRef->{$items}->{value};
    my $itemDescription = $hashRef->{$items}->{description};
    my $isArray = $hashRef->{$items}->{array};

    if($menuNumber == $thisVar)
    {
      print "$itemDescription\n\n";
      print "$menuItem [$defaultValue]: ";
      $selection = <STDIN>;
      chop $selection;
      $change = "true";
      if($selection ne "")
      {
        if($isArray == 1)
        {
          $hashRef->{$items}->{value} = "array(";
          my @splitArray = split(/,/,$selection);
          foreach $value(@splitArray)
          {
            $hashRef->{$items}->{value} .= "\"$value\",";
          }
          $hashRef->{$items}->{value} .= ")";
        }
        else
        {
          $hashRef->{$items}->{value} = $selection;
        }
        $change = "true";
      }
    }
  }
  return $change;
}

sub printMenu ($$)
{
  $hashRef = shift;
  $version = shift;
  print "Welcome to $version Setup\n";
  print "================================================================================\n\n";
  foreach my $items (sort { $hashRef->{$a}->{menuOrder} <=>
                          $hashRef->{$b}->{menuOrder} } (keys %{$hashRef}) )
  {
    my $menuNumber      = $hashRef->{$items}->{menuOrder};
    my $menuItem        = $hashRef->{$items}->{menuListing};
    my $defaultValue    = $hashRef->{$items}->{value};
    my $itemDescription = $hashRef->{$items}->{description};
    if($menuNumber < 10)
    {
      print " $menuNumber) $menuItem [$defaultValue]\n";
    }
    else
    {
      print "$menuNumber) $menuItem [$defaultValue]\n";
    }
  }
  print " s) Save and Quit\n";
  print " q) Quit Without Saving\n\n";
  print "Please make a selection: ";
}

sub configure()
{
  my $status = "active";
  my $dbVer = "dropbox 0.0.8Beta";
  my $selection = "";
  my $cmd = "";
  my $now = `date`;
  chop $now;
  
  my ($key, $value, $elname, $elvalue, $change, $configFile, @raw_data, %variables);
  $variables{basedir}{menuOrder} = 1;
  $variables{basedir}{menuListing} = $dbVer . ' Installation Dir';
  $variables{basedir}{value} = `pwd`;
  $variables{basedir}{array} = 0;
  chop $variables{basedir}{value};
  $variables{basedir}{description} = 
    "################################################################################\n" .
    "# This is your php-dropbox installation directory (i.e. the path where this\n" . 
    "# package is installed).  You need the complete path to this installation.  If\n" .
    "# you are unsure of this value, I suggest that you go to the spot where you \n" .
    "# installed the php-dropbox package and type:\n" .
    "#   pwd [enter]\n" .
    "# The resultant path is the value you want to enter here.\n" .
    "# \n";
  
  $variables{file_dir}{menuOrder} = 2;
  $variables{file_dir}{menuListing} = 'Uploads Directory';
  $variables{file_dir}{value} = $variables{basedir}{value} . '/uploads/';
  $variables{file_dir}{array} = 0;
  $variables{file_dir}{description} = 
    "################################################################################\n" .
    "# Where would you like user's uploaded files to go?  You will need to \n" .
    "# ensure that your www user has read/write/execute access to this \n" .
    "# directory\n" .
    "#  e.g. chmod 777 /path/to/uploads\n" .
    "# \n";
  
  $variables{wwwUploadDir}{menuOrder} = 3;
  $variables{wwwUploadDir}{menuListing} = 'WWW Uploads Base Directory';
  $variables{wwwUploadDir}{value} = './uploads/';
  $variables{wwwUploadDir}{array} = 0;
  $variables{wwwUploadDir}{description} = 
    "################################################################################\n" .
    "# Where will users go to access your uploads base directory?\n" .
    "#  e.g. ./uploads/\n" .
    "# \n";
  
  $variables{timeLimit}{menuOrder} = 4;
  $variables{timeLimit}{menuListing} = 'Upload time limit';
  $variables{timeLimit}{value} = 0;
  $variables{timeLimit}{array} = 0;
  $variables{timeLimit}{description} = 
    "################################################################################\n" .
    "# How much time do you want to allot to any give file upload.  '0' means\n" .
    "# that there is not limit which, provided that you are running the app\n" .
    "# password protected, is probably fine.  Feel free to experiment with this\n" .
    "# value, however.\n" .
    "# \n";

  $variables{numoffile}{menuOrder} = 5;
  $variables{numoffile}{menuListing} = 'Number of concurrent files a user can upload';
  $variables{numoffile}{value} = 5;
  $variables{numoffile}{array} = 0;
  $variables{numoffile}{description} = 
    "################################################################################\n" .
    "# How many files would you like to let a user upload in a single try? I\n" .
    "# have set the default at 5, but feel free to experiment with different\n" .
    "# values (higher or lower).\n" .
    "# \n";

  $variables{WebTitle}{menuOrder} = 6;
  $variables{WebTitle}{menuListing} = 'Page Title Listing';
  $variables{WebTitle}{value} = 'My Upload Page';
  $variables{WebTitle}{array} = 0;
  $variables{WebTitle}{description} = 
    "################################################################################\n" .
    "# What would you like user's to see as your php-dropbox title?\n" . 
    "# \n";


  $variables{fileData}{menuOrder} = 7;
  $variables{fileData}{menuListing} = 'Where would you like to store your file data?';
  $variables{fileData}{value} = './fileData.dat';
  $variables{fileData}{array} = 0;
  $variables{fileData}{description} = 
    "################################################################################\n" .
    "# We store upload information in a flat file, as of v0.0.2Beta. By default,\n" .
    "# this information is written to /path/to/dropbox/fileData.dat, which \n" .
    "# should suffice.  If you specify another file to use, just be certain that\n" .
    "# your www user has read and write access to that file.\n" .
    "# \n";


  $variables{sendMeMail}{menuOrder} = 8;
  $variables{sendMeMail}{menuListing} = 'Do you want an email to be sent upon file upload?';
  $variables{sendMeMail}{value} = 'no';
  $variables{sendMeMail}{array} = 0;
  $variables{sendMeMail}{description} = 
    "################################################################################\n" .
    "# If you would like to receive an email every time that a user uploads a\n" .
    "# file, set this to yes.  This _can_ be useful to see who is uploading \n" .
    "# what.\n" .
    "# \n";


  $variables{mailTo}{menuOrder} = 9;
  $variables{mailTo}{menuListing} = 'Email address to send to';
  $variables{mailTo}{value} = 'irb@ianbezanson.ca';
  $variables{mailTo}{array} = 0;
  $variables{mailTo}{description} = 
    "################################################################################\n" .
    "# If you had chosen to receive an email whenever someone uploads a file,\n" .
    "# what email address would you like to send it to?\n" .
    "#   Please note: I'd prefer that you don't leave this as the default :)\n" .
    "# \n";

  $variables{passwordProtect}{menuOrder} = 10;
  $variables{passwordProtect}{menuListing} = 'Would you like to password protect dropbox?';
  $variables{passwordProtect}{value} = 'yes';
  $variables{passwordProtect}{array} = 0;
  $variables{passwordProtect}{description} = 
    "################################################################################\n" .
    "# Do you want users to be prompted for a username/password when they come\n" . 
    "# to your dropbox?\n" .
    "#   default: yes\n" .
    "#   suggestion: YES\n" .
    "#   options: yes/no\n" .
    "# \n";

  $variables{htpasswdLoc}{menuOrder} = 11;
  $variables{htpasswdLoc}{menuListing} = 'Where is your htpasswd file located?';
  $variables{htpasswdLoc}{value} = $variables{basedir}{value} . '/.htpasswd';
  $variables{htpasswdLoc}{array} = 0;
  $variables{htpasswdLoc}{description} = 
    "################################################################################\n" .
    "# If you are password protecting your dropbox, as I'd imagine/hope you are\n" . 
    "# where is your .htpasswd file located?  Don't worry if you don't know, as\n" .
    "# if that is the case, just stick with the default.\n" .    
    "# \n";

  while ($status eq "active")
  {
    if ($status eq "active")
    {
      system("clear");
    }
    $configFile = $variables{basedir}{value} . "/config.php";
    $htAccessFile = $variables{basedir}{value} . "/.htaccess";

    if (-e $configFile && $change ne "true")
    {
      if (-w $configFile && -r $configFile)
      {
        open(DATA, $configFile) || die("Could not open file!");
        @raw_data=<DATA>;
        close(DATA);
        my $value;
        foreach $value (@raw_data)
        {
          chop($value);
          if($value =~ m/\<\?|\?\>/)
          {
            ;
          }
          else
          {
            ($elname,$elvalue)=split(/ = /,$value);
            $elname =~ s/^\$//g;
            $elvalue =~ s/\"//g;
            $elvalue =~ s/;$//g;
            $variables{$elname}{value} = $elvalue if defined $variables{$elname};
          }
        }
      }
      else
      {
        if (-w $configFile)
        {
          system("clear");
          print "$configFile is not readable!\nPlease `chmod 777 $configFile`\n";
          exit 0;
        }
        else
        {
          system("clear");
          print "$configFile is not writeable!\nPlease `chmod 777 $configFile`\n";
          exit 0;
        }
      }
    }
    else
    {
      if (-e "$variables{basedir}{value}")
      {
        if (-w "$variables{basedir}{value}" && -r "$variables{basedir}{value}")
        {
          ;
        }
        else
        {
          if (-w "$variables{basedir}{value}")
          {
            system("clear");
            print "$variables{basedir}{value} is not readable!\nPlease `chmod 777 $variables{basedir}{value}`\n";
            exit 0;
          }
          else
          {
            system("clear");
            print "$variables{basedir}{value} is not writeable!\nPlease `chmod 777 $variables{basedir}{value}`\n";
            exit 0;
          }
        }
      }
    }

    printMenu (\%variables, $ver);
    $selection = <STDIN>;
    chop $selection;

    if($selection >= 1 && $selection <= keys(%variables))
    {
      $change = setValue (\%variables, $selection);
    }
    elsif($selection eq "s" || $selection eq "S")
    {
      $status = "quit";
      system("clear");
      open(DATA,">$configFile") || die("Cannot Open $configFile");
      print DATA "<?php\n";
      print DATA "################################################################################\n";      
      print DATA "#  " . $dbVer . " Configuration File\n";
      print DATA "# \n";
      print DATA "#  Configuration File AutoGenerated on: $now\n";
      print DATA "################################################################################\n";      
      print DATA "# Dropbox Version";
      print DATA "# \n";
      print DATA "\$dbVer = \"$dbVer\";\n";
      print DATA "# \n";
      foreach my $items (sort { $hashRef->{$a}->{menuOrder} <=>
                          $hashRef->{$b}->{menuOrder} } (keys %{$hashRef}) )
      {
        my $value           = $hashRef->{$items}->{value};
        my $itemDescription = $hashRef->{$items}->{description};
        if($value =~ m/array/)
        {
          print DATA "$itemDescription";
          print DATA "\$$items = $value;\n";
          print DATA "# \n";
        }
        else
        {
          print DATA "$itemDescription";
          print DATA "\$$items = \"$value\";\n";
          print DATA "# \n";
        }
      }
      print DATA "?>\n";
      close(DATA);
      
      if ($variables{passwordProtect}{value} eq "yes")
      {
        open(DATA,">$htAccessFile") || die("Cannot Open $htAccessFile");
        print DATA "################################################################################\n";      
        print DATA "#  " . $dbVer . " .htaccess File\n";
        print DATA "# \n";
        print DATA "#  .htaccess File AutoGenerated on: $now\n";
        print DATA "\n";
        print DATA "AuthName \"$variables{WebTitle}{value}\"\n";
        print DATA "AuthType Basic\n";
        print DATA "AuthUserFile $variables{htpasswdLoc}{value}\n";
        print DATA "require valid-user\n";
        close(DATA);
        print "Please `chmod 755 $variables{basedir}{value} && chmod 644 $configFile && chmod 644 $htAccessFile && chmod 777 $variables{file_dir}{value} && chmod 666 $variables{fileData}{value}` for added security!\n";
      }
      else
      {
        if (-e $htAccessFile)
        {
          exec(`rm $htAccessFile`);
          print "Please `chmod 755 $variables{basedir}{value} && chmod 644 $configFile && chmod 777 $variables{file_dir}{value} && chmod 666 $variables{fileData}{value}` for added security!\n";
        }
      }
    }
    elsif($selection eq "q" || $selection eq "Q")
    {
      $status = "quit";
      print "Are you sure you want to quit without saving [yes/no]? ";
      while($selection ne "yes" && $selection ne "no")
      {
        $selection = <STDIN>;
        chop $selection;
        if($selection eq "yes")
        {
          system("clear");
        }
        elsif($selection eq "no")
        {
          $status = "active";
          system("clear");
        }
        else
        {
          print "Please enter 'yes' or 'no': ";
        }
      }
    }
    else
    {
      $status = "active";
    }
  }
}

configure();
